# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.


__maintainer__ = 'Benjamin Kampmann <benjamin@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider
from elisa.core.media_uri import MediaUri
from elisa.core import common, component
import os
from twisted.internet import defer, threads

from elisa.extern.flickrest import Flickr, FlickrError
from twill import commands as twill


from elisa.extern.translation import gettexter, N_
T_ = gettexter('elisa-flickr')

plugin_registry = common.application.plugin_registry
InternetLocationMessage = plugin_registry.get_component_class('base:internet_location_message')

class FlickrMedia(MediaProvider):
    """
    DOCME
    """
    secret = '14693e87a1349c20'
    key = '4bb9b930c2f01fab6b3fd20e02d165fb'

    config_doc = {'per_page' :  'show so many entries should be' \
                                'shown per page?',
                  'login': 'Your Flickr username (optional)',
                  'password': 'Your Flickr password (optional)',
                  'locations': 'A list of tags or flickr:// URIs'
                  }


    default_config = {'per_page': '30',
                      'login': '',
                      'password': '',
                      'locations': []
                      }


    def initialize(self):
        MediaProvider.initialize(self)
        self._cache = {}
        self._login = self.config.get('login','')
        self._password = self.config.get('password','')
        self._per_page = int(self.config.get('per_page', '30'))
        self._locations = self.config.get('locations', [])
        
        self._show_pages = False # Hardcoded, but I'll be a config-option
                                 # when it is usable
        self._connected = False
        self._perm = "read"

        if self._login and self._password:
            self._perm = "write"

        try:
            self._flickr = Flickr(self.key, self.secret, self._perm)
            if self._perm == "write":
                threads.deferToThread(self._connect_flickr)
            else:
                self._connected_cb(None)
        except FlickrError, e:
            raise component.InitializeFailure(self.name, e.get('msg'))


    def _connected_cb(self, auth):
        self.debug("Connected, %r", auth)
        self._connected = True

        uri = "flickr:///"
        action_type = InternetLocationMessage.ActionType.LOCATION_ADDED
        msg = InternetLocationMessage(action_type, "flickr", 'flickr', uri,
                                           media_types=['image',],
                                           theme_icon='flickr')
        common.application.bus.send_message(msg)

    def _error_cb(self, failure):
        self.warning('Failed to connect to flickr. Maybe the API-Key or the'
                     ' Secret is wrong. Error message :%s',
                     failure.getErrorMessage())

    def _connect_flickr(self):

        def auth_open_url(state):
            if state is None:
                self._connected_cb(state)
            else:
                twill.go(state['url'])
                twill.formvalue(1, 'login', self._login)
                twill.formvalue(1, 'passwd', self._password)
                twill.submit()
                twill.config('readonly_controls_writeable', 'True')
                try:
                    twill.formvalue(3, 'done_auth', '1')
                    twill.submit()
                except:
                    # already granted
                    pass
                dfr = self._flickr.authenticate_2(state)
                dfr.addCallbacks(self._connected_cb, self._error_cb)
                return dfr

        self.debug("Authenticating")
        self._flickr.authenticate_1().addCallbacks(auth_open_url,
                                                   self._error_cb)

    def scannable_uri_schemes__get(self):
        return []

    def supported_uri_schemes__get(self):
        return {'flickr' : 0}

    def get_real_uri(self, uri):
        if self._blocking_is_directory(uri):
            return uri

        server = uri.get_param('server')
        secret = uri.get_param('secret')
        # remove the starting '/'
        id = uri.path[1:]
        # build the real URL for the Image
        return MediaUri(self._build_uri(id, server, secret, 'medium'))

    def _build_uri(self, id, server, secret, size):
        size_suffixes = {'square': '_s',
                         'thumbnail': '_t',
                         'small': '_m',
                         'medium': '',
                         'large': '_b',
                         'original': '_o',
                        }
        uri = "http://static.flickr.com/%s/%s_%s%s.jpg" % (server, id, secret,
                                                           size_suffixes[size])
        return uri

    def get_media_type(self, uri):
        return defer.succeed(self._blocking_get_media_type(uri))

    def _blocking_get_media_type(self, uri):
        if self._blocking_is_directory(uri):
            return {'file_type' : 'directory', 'mime_type' : ''}
        else:
            return {'file_type' : 'image', 'mime_type' : 'image/jpeg'}


    def is_directory(self, uri):
        return defer.succeed(self._blocking_is_directory(uri))

    def _blocking_is_directory(self, uri):
        return uri.host != "photo"

    def has_children_with_types(self, uri, media_types):
        df = threads.deferToThread(self._blocking_has_children_with_types, uri,
                                   media_types)
        return df

    def _blocking_has_children_with_types(self, uri, media_types):
        has_children = False
        media_type_ok = len(set(['image',
                                 'directory']).intersection(media_types)) > 0
        if uri.host != 'photo' and media_type_ok:
            has_children = True
        return has_children


    def get_direct_children(self, uri, children_with_info):
        if not self._connected:
            dfr = defer.Deferred()
            dfr.callback(children_with_info)

        elif uri.host == '':
            return self._build_main_menu(children_with_info)
        else:
            dfr = self._dispatch(uri, children_with_info)

        return dfr


    def next_location(self, from_uri, root=None):
        next_uri = None
        
        def got_children(children):
            self._cache[from_uri] = [ str(i[0]) for i in children ]
            return MediaUri(self._cache[from_uri][0])
        
        if self._blocking_is_directory(from_uri):
            dfr = self.get_direct_children(from_uri,[])
            dfr.addCallback(got_children)
        else:
            for dir_uri, items in self._cache.iteritems():
                try:
                    index = items.index(str(from_uri))
                except:
                    pass
                else:
                    try:
                        next_uri = MediaUri(items[index+1])
                    except:
                        pass
                    else:
                        break
            if next_uri:
                if self._blocking_is_directory(next_uri):
                    dfr = self.next_location(next_uri, root)
                else:
                    dfr = defer.succeed(next_uri)
            else:
                dfr = defer.succeed(None)
        return dfr
    

    def _dispatch(self, uri, children_with_info):
        page = int(uri.get_param('page', '0'))
        
        if uri.host == 'mine':
            if uri.path[1:] == 'latest':
                dfr = self._flickr.photos_search(user_id='me', page=page,
                                                 per_page=self._per_page)
                dfr.addCallback(self._got_images, children_with_info, str(uri))

            elif uri.path[1:] == 'photosets':
                dfr = self._flickr.photosets_getList(per_page=self._per_page)
                dfr.addCallback(self._got_photosets, children_with_info)

            elif uri.path[1:] == "tags":
                dfr = self._flickr.tags_getListUser(per_page=self._per_page)
                dfr.addCallback(self._got_whotags, children_with_info)

            elif uri.path[1:] == 'favorites':
                dfr = self._flickr.favorites_getList(page=page)
                dfr.addCallback(self._got_images, children_with_info,str(uri))

            else:
                dfr = threads.deferToThread(self._build_mine_menu,children_with_info)

        elif uri.host == 'photo':
            dfr = defer.Deferred()
            dfr.callback(children_with_info)

        elif uri.host == 'photoset':
            dfr = self._flickr.photosets_getPhotos(photoset_id=uri.path[1:],
                                                   page=page,
                                                   per_page=self._per_page)
            dfr.addCallback(self._got_images_in_photoset, children_with_info,
                            str(uri))

        elif uri.host == 'interesting':
            dfr = self._flickr.interestingness_getList(page=page,
                                                       per_page=self._per_page)
            dfr.addCallback(self._got_images, children_with_info, str(uri))

        elif uri.host == "hottags":
            dfr = self._flickr.tags_getHotList()
            dfr.addCallback(self._got_hottags, children_with_info)

        elif uri.host == "tag":
            who = uri.get_param('who')
            dfr = self._flickr.photos_search(tags=uri.path[1:],
                                             user_id=who,
                                             page=page,
                                             per_page=self._per_page)
            dfr.addCallback(self._got_images, children_with_info, str(uri))
            
        elif uri.host == "custom":
            
            for location in self._locations:
                children_with_info.append((MediaUri(location), {}))
            dfr = defer.succeed(children_with_info)
        else:
            dfr = defer.succeed(children_with_info)
        return dfr

    def _build_mine_menu(self, list_of_children):

        mine_menu  = {"flickr://mine/favorites" : T_(N_("my Favorites")),
                      "flickr://mine/latest" : T_(N_("latest")),
                      "flickr://mine/photosets" : T_(N_("in photosets")),
                      "flickr://mine/tags" : T_(N_("by tag"))}

        return self._from_list(list_of_children, mine_menu)


    def _build_main_menu(self, list_of_children):

        # the interesting uri
        interesting = MediaUri("flickr://interesting/")

        # create the main-menu
        main_menu = {"flickr://hottags/": T_(N_("Hot Tags")),}
        if self._perm == 'write':
            main_menu["flickr://mine/latest"] = T_(N_("My Photos"))
        if self._locations:
            main_menu["flickr://custom/"] = T_(N_("Custom locations"))
            
        self._from_list(list_of_children, main_menu)

        # fetch interesting images for the main-menu, too
        return self._dispatch(interesting, list_of_children)

    def _from_list(self, list_of_children, menu):

        for entry, label in menu.items():
            list_of_children.append((self._make_uri(entry, label), {}))


        return list_of_children

    def _make_uri(self, uri, label=None):
        m = MediaUri(uri)
        if label != None:
            m.label = label
        return m

    def _got_images(self, ret, list_of_children, url=""):
        photos = ret.find('photos')
        pages = int(photos.get('pages'))
        page = int(photos.get('page'))

        for nu in ret.findall("photos/photo"):
            self._add_photo(nu, list_of_children)

        # We don't support more pages yet
        if (pages > 1 and page == 1 and self._show_pages):
            i = 1
            while (i < pages):
                i = i + 1
                m = MediaUri(url)
                m.set_param('page', i)
                m.label = T_(N_('page %s'), i)
                list_of_children.append((m,{}))
        return list_of_children

    def _got_images_in_photoset(self, ret, list_of_children, url=''):
        photos = ret.find('photoset')
        pages = int(photos.get('pages'))
        page = int(photos.get('page'))

        for nu in ret.findall("photoset/photo"):
            self._add_photo(nu, list_of_children)

        #  We don't support more pages before the GUADEC 07
        if (pages > 1 and page == 1 and self._show_pages):
            i = 1
            while (i < pages):
                i = i + 1
                m = MediaUri(url)
                m.set_param('page', i)
                m.label = T_(N_('page %s'), i)
                list_of_children.append((m,{}))
        return list_of_children

    def _add_photo(self, nu, list_of_children):
        id = nu.get('id')
        title = nu.get('title')
        server = nu.get('server')
        secret = nu.get('secret')

        uri = self._make_uri("flickr://photo/%s?server=%s&secret=%s" %
                                    (id, server, secret), title)
        list_of_children.append((uri, {
                'title' : title,
                'server' : server,
                'secret' : secret,
                'owner' : nu.get('owner'),
                'ispublic' : nu.get('ispublic'),
                'isfriend' : nu.get('isfriend'),
                'isfamily' : nu.get('isfamily'),
                'default_image': MediaUri(self._build_uri(id, server, secret,
                                                          'small')
                                         )
                                        }
                                    )
                                )
        return list_of_children

    def _got_hottags(self, ret, list_of_children):
        for nu in ret.findall("hottags/tag"):
            name = nu.text
            score = nu.get('score')
            uri = MediaUri("flickr://tag/%s" % name)
            list_of_children.append(( uri, {"name" : name, score: "score"}))
        return list_of_children

    def _got_whotags(self, ret, list_of_children):
        who = ret.find("who").get('id')
        for nu in ret.findall("who/tags/tag"):
            name = nu.text
            score = nu.get('score')
            uri = MediaUri("flickr://tag/%s?who=%s" % (name, who))
            list_of_children.append(( uri, {"name" : name, score: "score"}))
        return list_of_children

    def _got_photosets(self, ret, list_of_children):
        for nu in ret.findall("photosets/photoset"):
            id = nu.get('id')
            name = nu.find('title').text
            server = nu.get('server')
            photos = nu.get('photos')
            secret = nu.get('secret')
            primary = nu.get('primary')
            uri = self._make_uri("flickr://photoset/%s" % id, name)
            list_of_children.append((uri, { 'name' : name,
                                            'id' : id,
                                            'server' : server,
                                            'photos' : photos,
                                            'secret' : secret,
                                            'primary' : primary}
                                            )
                                    )
        return list_of_children

    



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

"""
StageMedia component class
"""

__maintainer__ = 'Florian Boucault <florian@fluendo.com>'
__maintainer2__ = 'Benjamin Kampmann <benjamin@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider, UriNotMonitorable
from elisa.core.media_uri import MediaUri, unquote
from elisa.core.utils import deferred_action
from elisa.core import common

from elisa.extern.configobj import Section

from elisa.core.observers.dict import DictObservable

from twisted.internet import defer, threads
from twisted.internet import reactor
from twisted.web import client
#from twisted.web.client import HTTPDownloader

from BeautifulSoup import BeautifulSoup

import urllib2, urllib, cookielib
import re
from mutex import mutex

from elisa.extern.translation import gettexter, N_
T_ = gettexter('elisa-stage6')


plugin_registry = common.application.plugin_registry
InternetLocationMessage = plugin_registry.get_component_class('base:internet_location_message')

class StageParser:

    """
    This class implements a parser to retrieve video titles and
    URL from a Stage6 HTML page
    """

    # Some regexps that will help retrieve the data we are looking
    # for in HTML pages
    reg_href = re.compile("href=\"(.*)\"")
    reg_href_avatar = re.compile("href=\"(.*)\"><acronym")
    reg_img = re.compile("alt=\"(.*)\" src=\"(.*)\"")
    reg_time = re.compile("<img (.*)/></acronym>(.*)")
    reg_img_avatar = re.compile("src=\"(.*)\" alt=")
    reg_img_title = re.compile("<acronym title=\"(.*)\" class=\"no-border\">")
    reg_title = re.compile("title=\"(.*)\">(.*)</a></")
    reg_video_id = re.compile("video/(.*)/")
    reg_pages = re.compile(">(.*)</a>")
    reg_watch_type = re.compile("<div class=\"user-watch\" id=\"(.*)\">")

    def __init__(self, string_to_parse):
        """
        @param string_to_parse:         the HTML code to parse
        @type string_to_parse:          string
        """

        self._to_parse = string_to_parse


    def get_tags(self):
        """
        Returns a list of tags as strings. This is parsing the
        HTML data we have in self._to_parse

        @rtype: list of strings
        """

        tags = []
        # In case the Stage6 website is having two many connections
        could_not_connect = '<!-- Could not connect to MySQL host: Too many connections -->'
        if self._to_parse.startswith(could_not_connect):
            tags.append("Could not connect")
            return tags

        # BeautifulSoup is going to help us find the code we're looking for
        b = BeautifulSoup(self._to_parse)
        res = b.findAll('ul', attrs={'class': 'tags-drill'})

        if len(res):
            # Tag names are between <li> marks
            res = res[0].findAllNext('li')
            for tag in res:
                t = tag.contents[0]
                for i in t.attrs:
                    if i[0] == 'class':
                        title = t.attrs[0][1]
                        tags.append(title)
                        break

        return tags

    def get_pages(self):
        """
        Returns an integer representing the last page. If there are no pages
        we return a zero
        """
        b = BeautifulSoup(self._to_parse)

        res = b.findAll('a', attrs={'class' : 'pagination-number'})
        if len(res) == 0:
            return 0

        last_page = self.reg_pages.search(str(res[len(res)-1])).groups()[0]

        return last_page

    def get_watchlist(self):
        """
        Returns a list of dictionaries, which look like this:
        {'label' : '', 'href' : '', 'img': ''}
        """

        b = BeautifulSoup(self._to_parse)

        res = b.findAll('div', attrs={'class' : 'user-watch'})


        list = []

        for div in res:
            type = self.reg_watch_type.search(str(div)).groups()[0]
            avatar = div.find('div', attrs={'class': 'avatar'})
            link = self.reg_href_avatar.search(str(avatar)).groups()[0]
            avatar_img = self.reg_img_avatar.search(str(avatar)).groups()[0]
            title = self.reg_img_title.search(str(avatar)).groups()[0]

            if type.startswith('user'):
                link = "%s/videos/group:uservideos" % link
            else:
                link = "%s/videos/" % link
               
            list.append({'label' : unquote(title), 'href' : link, 'img' : avatar_img})
        
        return list

    def get_videos(self):
        """
        Returns a list of videos, with their name, URL and thumbnail. This is parsing the
        HTML data we have in self._to_parse

        @rtype : list of (string, string, string)
        """

        videos = []

        # Video info are between <div> marks with their class='video-title'
        b = BeautifulSoup(self._to_parse)
        resu = b.findAll('div', attrs={'class': 'video'})

        for res in resu:
            vid_tit = res.find('div', attrs={'class' : 'video-title'})
            href = res.findAll('img')
            href = href[len(href)-1]
            if not vid_tit:
                continue

            line = str(vid_tit)
            overlay = str(res.find('div', attrs={'class' : 'video-overlay'}))
            times = self.reg_time.search(overlay)
            if times:
                time = times.groups()[1]
            else:
                time = None



            # get the href to retrieve video id
            match = self.reg_href.search(line)
            if not match:
                continue

            # retrieve the thumbnail location
            imgp = self.reg_img.search(str(href))
            if imgp:
                img = imgp.groups()[1]
            else:
                img = ''
            
            video_id = self.reg_video_id.search(match.groups()[0])

            if video_id:
                title = self.reg_title.search(line)
                if title:
                    # Finally we add the video id, its title and thumbnail location
                    videos.append((video_id.groups()[0],
                                   title.groups()[1],
                                   img, time))

        return videos


"""
class Downloader(HTTPDownloader):

    def pageEnd(self):

        print "got file"
"""


class StageMedia(MediaProvider):
    """
    This class implements stage6 video website browsing support
    http://stage6.divx.com
    """

    # URL where we are going to look for video tags
    MAIN_URL = "http://stage6.divx.com/"
    TAGS_URL = MAIN_URL + "videos/"
    # URL where the videos are actually located
    VIDEOS_URL = "stage6://video.stage6.com/"

    config_doc = {'email' : 'The email which is registered at stage6, optional',
                  'password' : 'the password',
                  'pages' : "Show the 'other pages'-item, experimental. See"
                            " CONFIGURATION for more informations",
                  'certain_uris': "This is a list of other Stage6-Uris, see"
                                  " CONFIGURATION in the plugins-directory for"
                                  " more informations.",
                  'max_pages' : "The maximum number of other pages to show"
                  }

    default_config = {'email' : '',
                      'password' : '',
                      'pages' : 0,
                      'max_pages' : 99,
                      'certain_uris': []
                      }
    

    def __init__(self):
        """
        We init here also the base class. Caching of directory listng
        will done in the self._cache dict, protected from concurrent
        access by a mutex
        """

        MediaProvider.__init__(self)

        # We create a cache of retrieved results
        self._cache = {}
        self._mutex = mutex()
        self._loggedIn = False
        self._more_pages = 0
        self._max_pages = 99
        self._certain_uris = {}

        # Here we create a DeferredActionManager, which permits us
        # to manage a queue of deferred actions. This is useful
        # for providers which uses a data protocol which can take a
        # lot of resources, in order to have only one request at a time
        self._def_action = deferred_action.DeferredActionsManager()
        
        self._cookie = cookielib.LWPCookieJar()
        # SetUp CookieStuff
        opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self._cookie))
        urllib2.install_opener(opener)


    def initialize(self):
        uri = "stage6:///"
        action_type = InternetLocationMessage.ActionType.LOCATION_ADDED
        msg = InternetLocationMessage(action_type, "Stage6", 'stage6', uri,
                                        media_types=['video',],
                                        theme_icon='stage6')
        common.application.bus.send_message(msg)
        self._more_pages = self.config.get('pages', 0)
        self._max_pages = self.config.get('max_pages', 99)
        uris = self.config.get('certain_uris', [])
        
        if isinstance(uris, list):
 
            for uri in uris:
                    self._certain_uris[uri] = None
            if isinstance(self.config, Section):
                for section in self.config.sections:
                    if section in self._certain_uris.keys():
                        pass
                        ## TODO: make this work!
#                    print type(section), section, dir(section)
#                    label = section.get('label', None)
#                    self._certain_uris[section] = label
        else:
            self.warning("Could not read configuration options certain_uris."
                            "Maybe it is not a list?")

        
        # The logIn should not block ;)
        return threads.deferToThread(self._logIn)


    def clean(self):
#        self.config.set('certain_uris', self._certain_uris)
        MediaProvider.clean(self)


    def _logIn(self):

        email = self.config.get('email', '')
        pw = self.config.get('password', '')

        if email != -1 and pw != '':
            
            self.debug("Loggin in with %s:%s" % (email, pw))
            data = urllib.urlencode({'email' : email,
                                     'password' : pw,
                                     'account' : 'true'})
            req = urllib2.Request('http://www.stage6.com/users/login/login-post',
                                     data)

            try:
                handler = urllib2.urlopen(req)
            except urllib2.URLError, u:
                self.warning("Could not log into stage6 server: %s" % u)
                return
            else:
                res = handler.read()
                if res.find('Log out') != -1:
                    self.debug("Logged In")
                    self._loggedIn = True
                    return

            self.warning("Could not log in. Maybe you should check your login"
                            " infromations")


    def scannable_uri_schemes__get(self):
        # We do not need media scanning. We can provide the metadata ourselves
        return {}

    def supported_uri_schemes__get(self):
        return { 'stage6': 0 }

    def get_media_type(self, uri):
        return self._def_action.insert_action(0, self._get_media_type, uri)

    def _get_media_type(self, uri):
        # If the uri starts with the stage6 video domain
        # name, we know it is a video. Otherwise it is
        # considered as a directory
        if repr(uri).startswith(self.VIDEOS_URL):
            return { 'file_type' : 'video',
                     'mime_type' : '' }
        else:
            return { 'file_type' : 'directory',
                     'mime_type' : '' }

    def is_directory(self, uri):
        return self._def_action.insert_action(0, self._is_directory, uri)

    def _is_directory(self, uri):
        # if the uri doesn't start with the stage6 video domain,
        # we know it is a directory
        return not repr(uri).startswith(self.VIDEOS_URL)

    def has_children(self, uri):
        return self._def_action.insert_action(0, self._has_children, uri)

    def _has_children(self, uri):
        # We can consider that a video tag we have found on the stage6
        # always have videos linked to.
        return self.is_directory(uri)

    def has_children_with_types(self, uri, media_types):
        return self._def_action.insert_action(0, self._has_children_with_types, uri, media_types)

    def _has_children_with_types(self, uri, media_types):
        if 'video' in media_types:
            return self._is_directory(uri)
        else:
            return False

    def _get_cached_uri(self, uri, children, add_info):
        """
        Return the list of children from a parent URI,
        or None if this URI has not yet been cached
        """
        self._mutex.testandset()

        ret = None
        # If we have the uri cached, return it
        if self._cache.has_key(repr(uri)):
            for i in self._cache[repr(uri)]:
                if add_info:
                    children.append(i)
                else:
                    children.append(i[0])
            ret = children

        self._mutex.unlock()

        return ret


    def _add_to_cache(self, parent, child, info):
        """
        Attach a child to a parent in the cache
        """

        self._mutex.testandset()

        parent = repr(parent)
        if not self._cache.has_key(parent):
            self._cache[parent] = [(child, info) ,]
        else:
            self._cache[parent].append((child, info))

        self._mutex.unlock()


    def get_real_uri(self, uri):
        # At this point we need to convert our internal stage6
        # uri to the real http uri that can be used to play a video
        # Fortunately, we just have to change the scheme.
        http = MediaUri(uri)
        http.scheme = 'http'
        self.info("The URI is %s" % http)
        return http


    def _read_url(self, url):
        """
        Read an URI and return its content
        """

        """
        dl = Downloader(url, "stage6.html")
        scheme, host, port, path = client._parse(url)
        reactor.connectTCP(host, port, dl)

        def download_complete(result):
            print "Download Complete."
            reactor.stop()

        def download_error(failure):
            print "Error:", failure.getErrorMessage()
            reactor.stop()

        dl.deferred.addCallback(download_complete).addErrback(download_error)
        """

        try:
            f = urllib2.urlopen(url)
        except urllib2.URLError, u:
            self.warning("Could not connect to stage6-server: %s" % u)
            return None
        return f.read()


    def _retrieve_children(self, uri, list_of_children, add_info=False):
        """
        retrieve the children of uri and fills list

        @param uri:                     the URI to analyze
        @type uri:                      L{elisa.core.media_uri.MediaUri}
        @param list_of_children:        List where the children will be appended
        @type list_of_children:         list
        @param add_info:                Add also the thumbnails to the list
        @type add_info:                 bool
        """

        # If the uri requested is in the cache, we return the cached children
        cache = self._get_cached_uri(uri, list_of_children, add_info)
        if cache:
            self.debug('Loaded from cache: %s' % repr(uri))
            return cache

        # if the uri path is /, we have to retrieve the tags from the main stage6 page
        if uri.path == '/':
            ### Make a main menu:

            uri = MediaUri('stage6:///videos/')
            uri.label = T_(N_("Featured Videos"))
            list_of_children.append((uri, {}))

            uri = MediaUri("stage6:///videos/order:hotness")
            uri.label = T_(N_("Hottest Videos"))
            list_of_children.append((uri, {}))

            uri = MediaUri("stage6:///videos/order:date")
            uri.label = T_(N_("Latest Videos"))
            list_of_children.append((uri, {}))

            uri = MediaUri("stage6:///videos/order:length")
            uri.label = T_(N_("Longest Videos"))
            list_of_children.append((uri, {}))

            uri = MediaUri("stage6:///tags")
            uri.label = T_(N_("Tags"))
            list_of_children.append((uri, {}))

            if self._loggedIn:
                uri = MediaUri("stage6:///me")
                uri.label = T_(N_("my WatchList"))
                list_of_children.append((uri, {}))

            if len(self._certain_uris):
                uri = MediaUri('stage6:///certains')
                uri.label = T_(N_("Certain Uris"))
                list_of_children.append((uri, {}))

            return list_of_children
           
        elif uri.path == '/tags':
            ### Make a main menu:
            uri = MediaUri('stage6:///tags/order:NoOrder')
            uri.label = T_(N_("Featured tags"))
            list_of_children.append((uri, {}))
            uri = MediaUri("stage6:///tags/order:hotness")
            uri.label = T_(N_("Hottest tags"))
            list_of_children.append((uri, {}))
            uri = MediaUri("stage6:///tags/order:date")
            uri.label = T_(N_("Latest tags"))
            list_of_children.append((uri, {}))
            uri = MediaUri("stage6:///tags/order:length")
            ## Does that make any sense?
            uri.label = T_(N_("Largest tags"))
            list_of_children.append((uri, {}))
            return list_of_children

        elif uri.path == '/certains':
            for uri,label in self._certain_uris.items():
                m = MediaUri(uri)
                if label != None:
                    m.label = label
                list_of_children.append((m, {}))

        # we have a list of tags        
        elif uri.path.startswith('/tags'):
            # We retrieve the HTML page
            path = uri.path[5:]
            to_parse = self._read_url(self.TAGS_URL + uri.path)
            if not to_parse:
                ## An Error happend:
                return list_of_children
            # create the parser and retrieve the tags
            parser = StageParser(to_parse)
            tags = parser.get_tags()

            # We add to the children list a MediaUri representing each tag
            for tag in tags:
                t = MediaUri("stage6:///videos/tag:%s%s" % (tag,path))
                t.label = tag

                if add_info:
                    list_of_children.append((t, {}))
                else:
                    list_of_children.append(t)
                # Cache the uri
                self._add_to_cache(uri, t, {})

        elif uri.path == "/me":
            url = "http://stage6.divx.com/user/any/watchlist/"
            to_parse =  self._read_url(url)
            if not to_parse:
                ### An Error happend:
                return list_of_children

            parser = StageParser(to_parse)            
            list = parser.get_watchlist()
            for item in list:
                uri = MediaUri("stage6://%s" % item['href'])
                uri.label = item['label']
                img = MediaUri(item['img'])
                list_of_children.append( (uri, {'default_image' : img}))
        elif uri.path.startswith('/pager'):
            pages = int(uri.get_param('pages', 0))
            # FIXME: how could this be done i18n ?
            pages_string = "Page %%%d.d" % len(str(pages))
            path = uri.path[7:]
            page_n = 2
            while page_n <= pages:
                page = MediaUri("stage6:///%s?page=%s" %
                                (path, page_n))
                page.label = pages_string % page_n
                list_of_children.append((page, {}))
                page_n +=1
        else:
            path = uri.path[1:] # Remove first slash
            page = uri.get_param('page', 1)
            url_path = "%s%s?page=%s" % (self.MAIN_URL, path, page)
            self.debug("Asking stage6 for: %s" % url_path) 
            # download HTML page and parse it to retrieve the video list
            to_parse = self._read_url(url_path)
            if not to_parse:
                self.warning("Reading didn't work")
                ### Error:
                return list_of_children
            parser = StageParser(to_parse)
            videos = parser.get_videos()
            self.debug("Found %s videos on that page" % len(videos))
            ## uncomment this line an comment the line beneath it, and the
            ## paging is enabled
            pages = 0
            if self._more_pages:
                pages = parser.get_pages()

            # We add to the children list a MediaUri representing each video
            for v in videos:
                t = MediaUri(self.VIDEOS_URL + v[0] + "/.avi")
                label = v[1].decode("utf-8")
                # set the uri label to the name of the video
                t.label = unquote(label)
                if add_info:
                    # Add the thumbnail url to the info dict
                    uri = v[2]
                    d = DictObservable()
                    if uri.startswith('http'):
                        d['default_image'] = MediaUri(uri)
                    if v[3] != None:
                        d['length'] = v[3]
                    list_of_children.append((t, d))
                    # Cache the uri
                    self._add_to_cache(uri, t, d)
                else:
                    list_of_children.append(t)
                    # Cache the uri
                    self._add_to_cache(uri, t, {})

            if pages > 1 and page == 1:
                if pages > self._max_pages:
                    pages = self._max_pages

                pager = MediaUri("stage6:///pager/%s?pages=%s"
                                    %(path, pages))
                pager.label = T_(N_("Other Pages"))
                list_of_children.append((pager, {}))

        return list_of_children

    def get_direct_children(self, uri, children_with_info):
        # Same as get_direct_children() except we also fill an information dict
        return self._def_action.insert_action(0, self._retrieve_children, uri, children_with_info, add_info=True)


    def next_location(self, uri, root=None):
        return self._def_action.insert_action(0, self._next_location, uri, root=root)

    def _next_location(self, uri, root=None):
        if not root:
            root_str = MediaUri(u'stage6:///')
        else:
            root_str = repr(root)

        to_find = repr(uri)
        # is it cached ?
        if self._cache.has_key(root_str):
            for child, children in self._cache.iteritems():
                # look if it is a child of root
                if child.startswith(root_str):
                    i = 0
                    while i < len(self._cache[child]):
                        # Is that our uri ?
                        if to_find == self._cache[child][i]:
                            # Check if there is a uri following
                            i += 1
                            if i < len(self._cache[child]):
                                # if yes, returns it
                                return MediaUri(self._cache[child][i])
                            break
                        i += 1

        return None

    def previous_location(self, uri):
        return None

    def uri_is_monitorable(self, uri):
        # We cannot monitor the uri for a change
        return False

    def uri_is_monitored(self, uri):
        # Always cannot be monitored
        return False

    def open(self, uri, mode=None, block=True):
        # We cannot open 'tags'
        if self.is_directory(uri):
            return None

        # What we do here is convert the uri in its http form,
        # and ask the media_manager to provide a suitable component
        # - such as GnomeVFSProvider - to do the work for us
        uri = self.get_real_uri(uri)
        media_manager = common.application.media_manager
        if media_manager.enabled:
            media = media_manager.open(uri, mode, block)
        else:
            media = None
        return media




# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

__maintainer__ = 'Philippe Normand <philippe@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider, UriNotMonitorable
from elisa.core.media_uri import MediaUri
from elisa.core import common
from elisa.core.observers.dict import DictObservable
from twisted.internet import defer, threads

import picasa_client

from elisa.extern.translation import gettexter, N_
T_ = gettexter('elisa-picasa')

class PicasaMedia(MediaProvider):
    """
    """
    default_config = {'username': '',
                      'password': ''}

    config_doc = {'username': 'Your Picasa username',
                  'password': 'Your password (needed to access private content)',
                  }
        
    def initialize(self):
        username = self.config.get('username', '')
        password = self.config.get('password', '')
        self._client = picasa_client.PicasaClient(username, password)
        
        plugin_registry = common.application.plugin_registry
        InternetLocationMessage = plugin_registry.get_component_class('base:internet_location_message')
        
        uri = "picasa:///"
        action_type = InternetLocationMessage.ActionType.LOCATION_ADDED
        msg = InternetLocationMessage(action_type, "Picasa", 'picasa', uri,
                                      media_types=['image',],
                                      theme_icon='picasa')
        common.application.bus.send_message(msg)
        self.debug("Initialize done")
        
    def scannable_uri_schemes__get(self):
        return {}
    
    def supported_uri_schemes__get(self):
        return { 'picasa': 0 }

    def get_media_type(self, uri):
        dfr = defer.Deferred()
        if self._is_directory(uri):
            dfr.callback({'file_type': 'directory', 'mime_type': ''})
        else:
            # XXX: don't hardcode
            dfr.callback({'file_type': 'image', 'mime_type': 'image/jpeg'})
        return dfr

    def is_directory(self, uri):
        d = defer.Deferred()
        d.callback(self._is_directory(uri))
        return d

    def _is_directory(self, uri):
        return uri.scheme == 'picasa' and uri.host != 'photo'

    def get_real_uri(self, uri):
        real_uri = None
        if uri.path[1:]:
            photo_id = "http%s" % uri.path[1:]
            real_uri = MediaUri(self._client.get_photo_url(photo_id))
        return real_uri

    def has_children_with_types(self, uri, media_types):
        dfr = defer.Deferred()
        
        has_children = False
        if self._is_directory(uri):
            if 'image' in media_types:
                has_children = True

        dfr.callback(has_children)
        return dfr

    def _build_main_menu(self, children):
        self.debug("Building main menu")
        
        child_uri = MediaUri('picasa://albums')
        child_uri.label = T_(N_("My albums"))
        children.append((child_uri, {}))

        child_uri = MediaUri('picasa://tags')
        child_uri.label = T_(N_("My tags"))
        children.append((child_uri, {}))
        
        return children
        
    def _retrieve_children(self, uri, children):
        photos = {}
        tags = {}
        albums = {}
        
        host = uri.host
        if host == '':
            children = self._build_main_menu(children)            
        elif host == 'albums':
            albums = self._client.get_albums()
        elif host == 'tags':
            tags = self._client.get_tags()
        elif host == 'album':
            album_id = uri.path[1:] # strip / from path
            album_id = "http%s" % album_id
            photos = self._client.get_photos_for_album_with_id(album_id)
        elif host == 'tag':
            tag_id = uri.path[1:] # strip / from path
            tag_id = "http%s" % album_id
            photos = self._client.get_photos_for_tag_with_id(tag_id)
            
        for album_id, label in albums.iteritems():
            album_id = MediaUri(album_id)
            album_id.scheme = ""
            child_uri = MediaUri('picasa://album/%s' % album_id)
            child_uri.label = label
            children.append((child_uri, {}))
            
        for tag_id, label in tags.iteritems():
            tag_id = MediaUri(tag_id)
            tag_id.scheme = ""
            child_uri = MediaUri('picasa://tag/%s' % tag_id)
            child_uri.label = label
            children.append((child_uri, {}))

        for photo_id, (label, thumbnail_url) in photos.iteritems():
            photo_id = MediaUri(photo_id)
            photo_id.scheme = ""
            child_uri = MediaUri('picasa://photo/%s' % photo_id)
            child_uri.label = label
            metadata = DictObservable()
            metadata['default_image'] = MediaUri(thumbnail_url)
            children.append((child_uri, metadata))
                
        return children

    def get_direct_children(self, uri, l):
        d = threads.deferToThread(self._retrieve_children, uri, l)
        return d

    def _blocking_open(self, uri, mode=None):
        return None

    def uri_is_monitorable(self, uri):
        return False



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import gdata.photos, gdata.photos.service
#import pdb; pdb.set_trace()
        
class PicasaClient:

    def __init__(self, username, password):
        self._pws = gdata.photos.service.PhotosService()
        self._pws.ClientLogin(username, password)

    def _get_photos_feed(self, feed_type):
        feed = self._pws.GetUserFeed(kind=feed_type).entry
        photo_feed = dict([ (a.id.text, a.title.text) for a in feed ])
        return photo_feed

    def _get_photos_from_entry(self, entry):
        photos = {}
        for photo in entry:
            #summary = photo.summary.text
            title = photo.title.text
            thumbnails = [ (t.width, t.height, t.url)
                           for t in photo.media.thumbnail]
            thumbnails = dict([((int(t.width), int(t.height)), t.url)
                               for t in photo.media.thumbnail])
            keys = thumbnails.keys()
            keys.sort()
            thumbnail = thumbnails[keys[-1]]
            photo_id = photo.id.text
            photos[photo_id] = (title, thumbnail)
            
        return photos

    def get_albums(self):
        return self._get_photos_feed('album')

    def get_tags(self):
        return self._get_photos_feed('tag')
        
    def get_photos_for_album_with_id(self, album_id):
        data = self._pws.Get(album_id).ToString()
        album = gdata.photos.AlbumEntryFromString(data)
        album_photos = self._pws.GetFeed(album.GetPhotosUri()).entry
        return self._get_photos_from_entry(album_photos)

    def get_photos_for_tag_with_id(self, tag_id):
        data = self._pws.Get(album_id).ToString()
        tag = gdata.photos.TagEntryFromString(data)
        tag_photos = self._pws.GetFeed(tag.GetTagssUri()).entry
        return self._get_photos_from_entry(tag_photos)
    
    def get_photo_url(self, photo_id):
        data = self._pws.Get(photo_id).ToString()
        photo = gdata.photos.PhotoEntryFromString(data)
        return photo.media.content[0].url
                
if __name__ == '__main__':
    clt = PicasaClient('', '')
    albums =  clt.get_albums()

    first = albums.keys()[0]
    
    photos = clt.get_photos_for_album_with_id(first)

    first_photo = photos.keys()[0]
    print clt.get_photo_url(first_photo)



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

__maintainer__ = 'Philippe Normand <philippe@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider, UriNotMonitorable
from elisa.base_components.media_provider import NotifyEvent
from elisa.core.component import InitializeFailure
from elisa.core.media_uri import MediaUri
from elisa.core import common
from elisa.core.observers.dict import DictObservable
from twisted.internet import defer, threads

from elisa.extern.translation import gettexter, N_
T_ = gettexter('elisa-youtube')


from youtube_client import YouTubeClient

import os, gst

plugin_registry = common.application.plugin_registry
InternetLocationMessage = plugin_registry.get_component_class('base:internet_location_message')

class YoutubeMedia(MediaProvider):
    """
    This class implements local filesystem support
    """
    default_config = {'user': ''}

    config_doc = {'user': 'Your Youtube username, optional'
                  }
        
    def initialize(self):
        try:
            gst.element_factory_make('flvdemux')
        except gst.PluginNotFoundError:
            msg = "Can't initialize flvdemux CVS version of gst-plugins-bad"
            raise InitializeFailure(self.name, msg)
        
        self._client = YouTubeClient()
        
        uri = "youtube:///"
        action_type = InternetLocationMessage.ActionType.LOCATION_ADDED
        msg = InternetLocationMessage(action_type, "YouTube", 'youtube', uri,
                                           media_types=['video',],
                                           theme_icon='youtube')
        common.application.bus.send_message(msg)
        self.debug("Initialize done")
        
    def scannable_uri_schemes__get(self):
        return {}
    
    def supported_uri_schemes__get(self):
        return { 'youtube': 0 }

    def get_media_type(self, uri):
        dfr = defer.Deferred()
        if self._is_directory(uri):
            dfr.callback({'file_type': 'directory', 'mime_type': ''})
        else:
            dfr.callback({'file_type': 'video', 'mime_type': 'video/x-flv'})
        return dfr

    def is_directory(self, uri):
        d = defer.Deferred()
        d.callback(self._is_directory(uri))
        return d

    def _is_directory(self, uri):
        return uri.scheme == 'youtube' and uri.host != 'video'

    def get_real_uri(self, uri):
        real_uri = None
        video_id = uri.path[1:]
        video_url = "http://www.youtube.com/watch?v=%s" % video_id
        real = self._client.get_flv_video_url(video_url)
        if real:
            real_uri = MediaUri(real)
        return real_uri

    def has_children_with_types(self, uri, media_types):
        dfr = defer.Deferred()
        
        has_children = False
        if self._is_directory(uri):
            if 'video' in media_types:
                has_children = True

        dfr.callback(has_children)
        return dfr

    def _build_main_menu(self, children):
        self.debug("Building main menu")
        child_uri = MediaUri('youtube://recently_featured/today')
        child_uri.label = T_(N_("Featured today"))
        children.append((child_uri, {}))

        child_uri = MediaUri('youtube://most_viewed')
        child_uri.label = T_(N_("Most viewed"))
        children.append((child_uri, {}))

        child_uri = MediaUri('youtube://top_rated')
        child_uri.label = T_(N_("Top rated"))
        children.append((child_uri, {}))

        child_uri = MediaUri('youtube://tags')
        child_uri.label = T_(N_("Tags"))
        children.append((child_uri, {}))

        user = self.config.get('user','')
        if user:
            child_uri = MediaUri('youtube://favorite_videos/%s' % user)
            child_uri.label = T_(N_("My favorite videos"))
            children.append((child_uri, {}))
            child_uri = MediaUri('youtube://user/%s' % user)
            child_uri.label = T_(N_("My videos"))
            children.append((child_uri, {}))
            child_uri = MediaUri('youtube://contacts/%s' % user)
            child_uri.label = T_(N_("My contacts"))
            children.append((child_uri, {}))
            
        return children
        

    def _retrieve_children(self, uri, children):
        videos = []
        users = []
        tags = {}
        
        host = uri.host
        if host == '':
            children = self._build_main_menu(children)
        elif host == 'contacts':
            username = uri.path[1:] # strip / from path
            users = self._client.get_contacts(username)
        elif host == 'recently_featured':
            videos = self._client.recently_featured()
        elif host == 'top_rated':
            videos = self._client.top_rated()
        elif host == 'most_viewed':
            videos = self._client.most_viewed()
        elif host == 'tags':
            tags = self._client.get_categories()
        elif host == 'tag':
            tagname = uri.path[1:] # strip / from path
            videos = self._client.get_category(tagname)
            
        elif host == 'favorite_videos':
            username = uri.path[1:] # strip / from path
            videos = self._client.favorite_videos(username)
        elif host == 'user':
            username = uri.path[1:] # strip / from path
            videos = self._client.videos_upload_by(username)

        for tag, label in tags.iteritems():
            child_uri = MediaUri('youtube://tag/%s' % tag)
            child_uri.label = label
            children.append((child_uri, {}))

        for video in videos:
            video_id = video.id.text.split("/")[-1]
            child_uri = MediaUri('youtube://video/%s' % video_id)
            child_uri.label = video.title.text
            thumbnail_url = self._client.get_largest_thumbnail(video)
            metadata = DictObservable()
            metadata['default_image'] = MediaUri(thumbnail_url)
            children.append((child_uri, metadata))
                
        if users is not None:
            for user in users:
                child_uri = MediaUri('youtube://user/%s' % user)
                child_uri.label = user
                children.append((child_uri, {}))
                
        return children


    def get_direct_children(self, uri, l):
        d = threads.deferToThread(self._retrieve_children, uri, l)
        return d

    def _blocking_open(self, uri, mode=None):
        return None

    def uri_is_monitorable(self, uri):
        return False



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

import gdata.service
import urllib2
import re
import traceback

try:
  from xml.etree import cElementTree as ElementTree
except ImportError:
  try:
    import cElementTree as ElementTree
  except ImportError:
    from elementtree import ElementTree


class YouTubeClient:

    users_feed = "http://gdata.youtube.com/feeds/users"
    std_feeds = "http://gdata.youtube.com/feeds/standardfeeds"
    video_name_re = re.compile(r', "t": "([^"]+)"')
    
    def _request(self, feed, *params):
        service = gdata.service.GDataService(server="gdata.youtube.com")
        return service.Get(feed % params)

    def recently_featured(self, time='today'):
        return self._request("%s/recently_featured", self.std_feeds).entry

    def top_rated(self):
        return self._request("%s/top_rated" % self.std_feeds).entry

    def most_viewed(self):
        return self._request("%s/most_viewed" % self.std_feeds).entry

    def videos_upload_by(self, username):
        return self._request("%s/%s/uploads", self.users_feed,
                             username).entry

    def favorite_videos(self, username):
        return self._request("%s/%s/favorites", self.users_feed,
                             username).entry

    def get_contacts(self, username):
        users = self._request("%s/%s/contacts", self.users_feed,
                              username).entry
        contacts = [ user.title.text for user in users ]
        return contacts
        
    def get_categories(self):
        feed_url = "http://gdata.youtube.com/schemas/2007/categories.cat"
        xml = self._request(feed_url)
        tree = ElementTree.fromstring(xml)
        categories = {}
        for child in tree.getchildren():
            categories[child.get('term')] = child.get('label')
        return categories
    
    def get_category(self, cid):
        feed_url = "http://gdata.youtube.com/feeds/videos/-/%s" % cid
        return self._request(feed_url).entry
        
    def get_thumbnails(self, video):
        doc = video._ToElementTree()
        urls = {}
        for c in doc.findall(".//{http://search.yahoo.com/mrss/}group"):
            for cc in c.findall("{http://search.yahoo.com/mrss/}thumbnail"):
                width = int(cc.get("width"))
                height = int(cc.get("height"))
                size = (width, height)
                url = cc.get("url")
                if size not in urls:
                    urls[size] = [url,]
                else:
                    urls[size].append(url)
        return urls

    def get_largest_thumbnail(self, video):
        thumbnails = self.get_thumbnails(video)
        sizes = thumbnails.keys()
        sizes.sort()
        return thumbnails[sizes[-1]][0]

    def get_flv_video_url(self, url):
        flv_url = ''
        doc = urllib2.urlopen(url)
        data = doc.read()

        # extract video name
        match = self.video_name_re.search(data)
        if match is not None:
            video_name = match.group(1)

            # extract video id
            url_splited = url.split("watch?v=")
            video_id = url_splited[1]     

            flv_url = "http://www.youtube.com/get_video?video_id=%s&t=%s"
            flv_url = flv_url % (video_id, video_name)
        return flv_url

if __name__ == '__main__':
    cli = YouTubeClient()
    import pdb; pdb.set_trace()



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

"""
Shoutcast Media Provider
"""


__maintainer__ = 'Benjamin Kampmann <benjamin@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider
from elisa.core.media_uri import MediaUri
from elisa.core import common

from elisa.extern.coherence.et import parse_xml

from twisted.internet import defer, threads

import urllib2, re

plugin_registry = common.application.plugin_registry
InternetLocationMessage = plugin_registry.get_component_class('base:internet_location_message')

# FIXME: use a non-blocking library to retrieve web pages (such as twisted.web
# or even simpler Elisa's media_manager)

class ShoutcastMedia(MediaProvider):
    """
    This class implements support for the shoutcast servers and playlist 
    """

    shout_uri = "http://www.shoutcast.com/sbin/newxml.phtml"
    pl_getter = re.compile('File.=(.*?)\n')

    intresting_genres = set([ 'Alternativ', "Classical", "Comedy", "Country",
                              "Dance", "Funk", "Jazz", "Metal", "Mixed",
                              "Pop", "Rap", "RnB", "Rock", "Talk", "Techno",
                              "80s", "70s", "World"])
    def initialize(self):
        self._all = []
        uri = "shoutcast:///"
        action_type = InternetLocationMessage.ActionType.LOCATION_ADDED
        msg = InternetLocationMessage(action_type, "Shoutcast",
                                            'shoutcast',
                                            uri,
                                            media_types=['audio',],
                                            theme_icon='shoutcast'
                                           )
        common.application.bus.send_message(msg)
        
    def _read_url(self, url):
        fread = urllib2.urlopen(url)
        return fread.read()
 
    def supported_uri_schemes__get(self):
        return { 'shoutcast': 0 }

    def scannable_uri_schemes__get(self):
        return []

    def _blocking_get_media_type(self, uri):
        if self._blocking_is_directory(uri):
            return {'file_type' : 'directory',
                    'mime_type' : 'mime_type'}

        return {'file_type' : 'audio', 'mime_type' : 'mp3'}

    def _blocking_is_directory(self, uri):
        if uri.host == 'PLAY':
            return False
        return True

    def get_media_type(self, uri):
        d = threads.deferToThread(self._blocking_get_media_type,
                                  uri)
        return d


    def has_children_with_types(self, uri, media_types):
        d = threads.deferToThread(self._blocking_has_children_with_types,
                                  uri, media_types)
        return d

    def _blocking_has_children_with_types(self, uri, media_types):
        if self._blocking_is_directory(uri):
            if 'audio' in media_types:
                return True
        return False

    def _blocking_get_direct_children(self, uri, list_of_children):

        if self._blocking_is_directory(uri):
            if uri.host == "ALL":
                for sub_uri in self._all:
                    list_of_children.append( (sub_uri, {}) )
            elif uri.path == '/':
                if len(self._all) == 0:
                    xml = parse_xml(self._read_url(self.shout_uri))
    
                    for elem in xml.findall('genre'):
                        name = elem.get('name')
                        uri = MediaUri('shoutcast:///%s' % name)
                        uri.label = name
                        self._all.append(uri)
                        if name in self.intresting_genres:
                            list_of_children.append( (uri, {}) )
                else:
                    for uri in self._all:
                        if uri.label in self.intresting_genres:
                            list_of_children.append( (uri, {}) )

                all_uri = MediaUri('shoutcast://ALL/')
                # FIXME: hardcoded is bad!
                all_uri.label = "All Genres"
                list_of_children.append( (all_uri, {}))
            else:
                # We have a genre
                genre = uri.path[1:]
                path = '%s?genre=%s' %(self.shout_uri, genre)
                xml = parse_xml(self._read_url(path))

                tune_in = xml.find('tunein')
                base = tune_in.get('base')

                for elem in xml.findall('station'):
                   name = elem.get('name')
                   id = elem.get('id')
                   uri = MediaUri('shoutcast://PLAY%s' % base)
                   uri.set_param('id', id)
                   uri.label = name

                   list_of_children.append( (uri, {}) )
                   if len(list_of_children) >= 20:
                       break

        return list_of_children

    def get_direct_children(self, uri, l):
        d = threads.deferToThread(self._blocking_get_direct_children, uri, l)
        return d

    def get_real_uri(self, uri):
        if self._blocking_is_directory(uri):
            return uri

        path = "http://www.shoutcast.com%s?id=%s" % (uri.path,
                                                     uri.get_param('id'))
        data = self._read_url(path)
        res = self.pl_getter.search(data)
        if res:
            link = res.groups()[0]
            new_uri = MediaUri(link)
            return new_uri
        return uri



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

"""
AudioScrobbler music stats submission
"""

__maintainer__ = "Philippe Normand <philippe@fluendo.com>"

from elisa.base_components import service_provider
from elisa.core import log, common, component, player
from elisa.core.observers.observer import Observer
from elisa.core.application import ComponentsLoaded
from twisted.internet import defer, threads

import gst
import urllib2, urllib, threading
import re, time, md5, os, socket
from cPickle import dump, load


plugin_registry = common.application.plugin_registry
PlayerModel = plugin_registry.get_component_class('base:player_model_message')

UNKNOWN = 'unknown'
SAVED_TRACKS_FILE = os.path.expanduser('~/.elisa/unsubmitted_tracks.txt')

socket.setdefaulttimeout(7)

class ScrobbledTrack:
    def __init__(self, media, length):
        if media.artist != 'unknown artist':
            self.artist = media.artist
        else:
            self.artist = UNKNOWN
        if media.album != 'unknown album':
            self.album = media.album
        else:
            self.album = UNKNOWN
        self.name = media.song
        self.length = str(length)

        # MusicBrainz not supported yet
        self.mbid = None
        
        self.date = re.sub("(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d).*","\\1 \\2",
                           time.strftime("%Y-%m-%dT%H:%M:%SZ",time.gmtime()))

    def isSubmitable(self):
        return UNKNOWN not in (self.artist, self.album)

    def __repr__(self):
        return "%r by %r from %r (%r @ %r)" % (self.name, self.artist,
                                               self.album,
                                               self.length, self.date)

    def urlencoded(self, num):
        encode = ""

        encode += "a["+str(num)+"]="+urllib.quote_plus(self.artist.encode('utf-8'))
        encode += "&t["+str(num)+"]="+urllib.quote_plus(self.name.encode('utf-8'))
        encode += "&l["+str(num)+"]="+urllib.quote_plus(self.length)
        encode += "&i["+str(num)+"]="+(self.date)
        if self.mbid is not None:
            encode += "&m["+str(num)+"]="+urllib.quote_plus(self.mbid)
        else:
            encode += "&m["+str(num)+"]="
        encode += "&b["+str(num)+"]="+urllib.quote_plus(self.album.encode('utf-8'))
        return encode

class PlayerModelObserver(log.Loggable, Observer):
    logCategory = "player_observer"
    
    def __init__(self, service):
        Observer.__init__(self)
        self._service = service
        self._submitted_uris = []
        self._current_uri = None
        
    def attribute_set(self, origin, key, old_value, new_value):
        self.log("New new_value for %r: %r", key, new_value)

        if key == 'position' and new_value > 0:
            player_model = self._observable

            uri = player_model.uri
            self._current_uri = uri
            
            if uri not in self._submitted_uris:
                length = player_model.duration / gst.SECOND
                status = new_value / gst.SECOND

                if status > length/2.:
                    media_manager = common.application.media_manager
                    media = media_manager.get_media_information(uri, extended=True)
                    self.debug("Found by media_manager: %r", media)
                    if media and hasattr(media, 'artist'):
                        self._submitted_uris.append(uri)
                        track = ScrobbledTrack(media, length)
                        threads.deferToThread(self._service.submit, track)
                else:
                    self.log("Not yet ready to report")
            else:
                self.log("Found new track, will report it soon")
        elif key == 'state' and new_value == player.STATES.STOPPED:
            current = self._current_uri
            if current is not None and current in self._submitted_uris:
                self.debug("Removing reference to %r", current)
                self._submitted_uris.remove(current)
            self._current_uri = None

class LastfmScrobbler(service_provider.ServiceProvider):
    """
    DOCME
    """

    config_doc = {'user': 'Last.FM username',
                  'password': 'Last.FM password for the user :-)'
                  }

    default_config = {'user': 'fill_me',
                      'password': 'fill_me'
                      }

    def initialize(self):
        user = self.config.get('user')
        if user == 'fill_me':
            msg = "Please configure your Last.FM account settings"
            raise component.InitializeFailure(self.name, msg)
        self.url = "http://post.audioscrobbler.com/"
        self.user = user
        self.password = self.config.get('password')
        self.client = "eli"
        self.version = "0.1"
        self.lock = threading.Lock()
        self.loadSavedTracks()

    def start(self):
        threads.deferToThread(self.handshake)
        common.application.bus.register(self._register_player_model,
                                        PlayerModel)

    def stop(self):
        self.saveTracks()

    def _register_player_model(self, msg, sender):
        self.debug("Got player model %r", msg.player_model)
        self._player_observer = PlayerModelObserver(self)
        msg.player_model.add_observer(self._player_observer)

    def loadSavedTracks(self):
        self.tracksToSubmit = []
        if os.path.exists(SAVED_TRACKS_FILE):
            f = open(SAVED_TRACKS_FILE, 'r')
            try:
                self.tracksToSubmit = load(f)
            finally:
                f.close()

    def saveTracks(self):
        if self.tracksToSubmit:
            self.debug('Saving %s tracks' % len(self.tracksToSubmit))
        f = open(SAVED_TRACKS_FILE, 'w')
        dump(self.tracksToSubmit, f)
        f.close()

    def handshake(self):
        self.logged = False
        self.debug("Handshaking...")
        url = self.url+"?"+urllib.urlencode({
            "hs":"true",
            "p":"1.1",
            "c":self.client,
            "v":self.version,
            "u":self.user
            })

        try:
            result = urllib2.urlopen(url).readlines()
        except urllib2.URLError, ex:
            self.debug("Could not connect to AudioScrobbler: %s", ex.reason.message)
        except Exception, ex:
            self.debug("Could not connect to AudioScrobbler: %s", ex)
        else:
            status = result[0]
            if status.startswith("BADUSER"):
                return self.baduser(result[1:])
            if status.startswith("FAILED"):
                return self.failed(result)
            self.logged = True
            if status.startswith("UPTODATE") or status.startswith("UPDATE"):
                return self.uptodate(result[1:])
            return True
        return False

    def uptodate(self, lines):
        self.md5 = re.sub("\n$","", lines[0])
        self.debug("MD5 = %r" % self.md5)
        self.submiturl = re.sub("\n$","", lines[1])
        self.debug("submiturl = %r" % self.submiturl)
        self.interval(lines[2])
        return True

    def baduser(self, lines):
        self.debug("Bad user")
        self.interval(lines[1])
        return False

    def failed(self, lines):
        self.debug('Failed : %s' % lines[0])
        self.interval(lines[1])
        return False

    def interval(self, line):
        match = re.match("INTERVAL (\d+)", line)
        if match is not None:
            secs = int(match.group(1))
            self.debug("Sleeping a while (%s seconds)" % secs)
            time.sleep(secs)

    def submit(self, track):
        if not self.logged:
            # try to log
            if not self.handshake():
                if track not in self.tracksToSubmit:
                    self.debug("Queued submission of track %s" % track)
                    self.tracksToSubmit.append(track)
                self.debug("%s track(s) currently queued" % len(self.tracksToSubmit))
                self.saveTracks()
                return

        if track not in self.tracksToSubmit:
            self.tracksToSubmit.append(track)

        self.debug("Will try to submit tracks : %s" % str(self.tracksToSubmit))

        try:
            md5response = md5.md5(md5.md5(self.password).hexdigest()+self.md5).hexdigest()
            self.debug('md5response: %s' % md5response)
            
            post = "u="+self.user+"&s="+md5response
            count = 0
            self.debug('post: %r' % post)
            
            for track in self.tracksToSubmit:
                l = int(track.length)
                if not track.isSubmitable():
                    self.debug("Missing informations from track, skipping submit")
                    self.debug(track)
                elif l < 30 or l > (30*60):
                    self.debug("Track is too short or too long, skipping submit")
                    self.debug(track)
                else:
                    self.debug('encoded: %r' % track.urlencoded(count))
                    post += "&"
                    post += track.urlencoded(count)
                    count += 1
        except Exception, ex:
            self.debug('Exception : %s' % ex)
        
        self.debug('count = %s' % count)
        self.debug(post)
        post = unicode(post)
        if count:
            try:
                result = urllib2.urlopen(self.submiturl,post)
            except Exception,ex:
                self.debug(ex)
                self.logged = False
            else:
                results = result.readlines()
                self.debug("submit result : %r" % results)
                if results[0].startswith("OK"):
                    self.interval(results[1])
                    self.tracksToSubmit = []
                    self.saveTracks()
                elif results[0].startswith("FAILED"):
                    self.failed(results)
                    self.handshake()
                elif results[0].startswith("BADAUTH"):
                    self.baduser(results)
                    self.handshake()



import sys, os


setup_path = os.path.abspath(__file__)
bundle_path= os.path.dirname(setup_path)
bundles_path = os.path.dirname(bundle_path)
plugins_path = os.path.dirname(bundles_path)
trunk_path = os.path.dirname(plugins_path)

try:
    from elisa.core.utils.dist import elisa_setup
except ImportError:
    core_path = os.path.join(trunk_path, 'elisa-core')
    sys.path.insert(0, core_path)
    if 'elisa' in sys.modules:
        del sys.modules['elisa']
    from elisa.core.utils.dist import elisa_setup

from elisa.core import __version__

name = 'elisa-plugins-ugly'
description = "Elisa plugins bad bundle"
long_description = description
license = "GPL3"
version = __version__
author = "Elisa Team"
author_email = "contact@fluendo.com"

plugins_dir = os.path.join(os.getcwd(), 'elisa-plugins')
if not os.path.isdir(plugins_dir):
    plugins_dir = os.path.join(trunk_path, 'elisa-plugins')

elisa_setup(name, description, long_description, version, license,
            author, author_email, 'elisa', plugins_dir)



# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

"""
This media provider provides access to the rss://-scheme and reads the
enclosures, if there are some or the describtion for images.

To use it follow these steps:
Get to your favorite podcast-website. copy the link of the rss, and paste it
into the locations of the audio activity for audio-data, in the video activity
for videos or in the image activity for images. Now remove the 'http' and
replace it with rss:
http://www.rocketboom.com/vlog/quicktime_daily_enclosures.xml
 = rss://www.rocketboom.com/vlog/quicktime_daily_enclosures.xml


This URI is now accessible in your audio (or video) menu inside browse by
folder.

Enjoy it!


Troubleshooting:
You get no Children, but it is not loading anymore?
    1. Are you sure that you entered in the right activity? Podcasts are
    NOT shown in video activity and the other way round.
    2. Are you sure, that the requested data is in there? To check this for
    videos/audios do this:
      Be sure that the rss you are using really has the enclosure tag. For this,
      you can just download the file and read it with your favorite editor.
      Look for the word 'enclosure'. If you don't find any, your blog does not
      support it and we can't read it.
    If you have problem with images, do this:
      You can just take a look with Firefox (> 2.0) at the file. If there are
      images appearing, this plugin should also be able to find them.

If you are sure, that it should work, but it doesn't: feel free to file a ticket
on the elisa trac. Don't forget to paste your link there and also, how you did
the troubleshooting!
"""


__maintainer__ = 'Benjamin Kampmann <benjamin@fluendo.com>'

from elisa.base_components.media_provider import MediaProvider
from elisa.core.media_uri import MediaUri, quote, unquote
from elisa.core.media_file import MediaFile

from elisa.extern.coherence.et import parse_xml

import urllib2, re

class PodcatcherMedia(MediaProvider):
    """
    This reads the enclosures for an rss-scheme...
    """

    reg_img_src = re.compile("<img.src=\"(.*?)\"")

    def __init__(self):
        MediaProvider.__init__(self)
        self._children = {}
        self._cached = []

    def scannable_uri_schemes__get(self):
        return {}

    def supported_uri_schemes__get(self):
        ### Feedparser also supports atom, so shouldn't we add it here?
        return { 'rss': 50 }

    def _blocking_get_media_type(self, uri):
        media_type = {}
        if self._blocking_is_directory(uri):
            media_type = {'file_type': 'directory', 'mime_type': ''}
        else:
            media_type = self._children.get(uri, None)
        return media_type

    def _blocking_is_directory(self, uri):
        return uri.host != 'CHILD'

    def _blocking_has_children_with_types(self, uri, media_types):
        has_children = False
        if self._blocking_is_directory(uri):
            ## we need a kind of a MAYBE-Opertor. We don't know, if there
            ## are this children, before accessing...
            has_children = True
        return has_children

    def _get_url_data(self, url):
        red = urllib2.urlopen(url)
        return red.read()

    def _blocking_get_direct_children(self, uri, children):
        if self._blocking_is_directory(uri) and not uri in self._cached:
            real_http = "http%s" % str(uri)[3:]
            self.debug("Real URI is %s" % real_http)

            data = self._get_url_data(real_http)

            parsed = parse_xml(data)

            for entry in parsed.findall('channel/item'):
                title = entry.find('title').text.lstrip()
                desc = entry.find('description')
                enclos = entry.find('enclosure')

                if enclos != None:
                    url = enclos.attrib['url']
                    length = enclos.attrib['length']
                    type = enclos.attrib['type']
                    media = None
                    if 'audio' in type:
                        media = 'audio'
                    elif 'video' in type:
                        media = 'video'
                    ### Is this also existing for images?
                    if url and media:
                        uri = MediaUri('rss://CHILD/%s' % quote(url))
                        uri.label = title
                        content = {'file_type' : media,
                                   'mime_type' : type,
                                   'default_image' : None,
                                   'length' : length} 
                        self._children[uri] = content
                        children.append( (uri, content) ) 
                elif desc != None:
                    # No real enclosures found. So let's search in the desc.
                    # currently only supported for images!
                
                    # The data should be quoted HTML.
                    data = unquote(desc.text)
                    data = data.replace("'","'")
                    search = self.reg_img_src.search(data)
                    if search != None:
                        for entry in search.groups():
                            content = {'file_type' : 'image',
                                       'media_type': '',
                                       'default_image' : MediaUri(entry) }

                            uri = MediaUri('rss://CHILD/%s' % quote(entry))
                            uri.label = title
                            self._children[uri] = content
                            children.append( (uri, content) )


        return children

    def get_real_uri(self, uri):
        return MediaUri(unquote(uri.path[1:]))


if __name__ == "__main__":

    m = PodcatcherMedia()
    uri = MediaUri('rss://www.brennpunkt-ostkreuz.de/rssfeed/Brennpunktgeschichten.rss')
    list =  []

    def printer (list):
        for i, meta in list:
            print "child", i, meta
            print "label", i.label
            print "real", m.get_real_uri(i)
        
    a = m.get_direct_children(uri, list)
    a.addCallback(printer)

    try:
        from twisted.internet import glib2reactor
        glib2reactor.install()
    except AssertionError:
        # already installed...
        pass

    from twisted.internet import reactor

    reactor.run()
